package ru.amse.fedorov.plainsvg.gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.List;

import javax.swing.Action;
import javax.swing.JPanel;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.actions.ChangeToolAction;
import ru.amse.fedorov.plainsvg.gui.actions.ZoomAction;
import ru.amse.fedorov.plainsvg.gui.actions.file.NewDocumentAction;
import ru.amse.fedorov.plainsvg.gui.actions.file.OpenDocumentAction;
import ru.amse.fedorov.plainsvg.gui.actions.file.SaveAsDocumentAction;
import ru.amse.fedorov.plainsvg.gui.actions.file.SaveDocumentAction;
import ru.amse.fedorov.plainsvg.gui.actions.move.MoveAction;
import ru.amse.fedorov.plainsvg.gui.actions.move.MoveBottomAction;
import ru.amse.fedorov.plainsvg.gui.actions.move.MoveDownAction;
import ru.amse.fedorov.plainsvg.gui.actions.move.MoveTopAction;
import ru.amse.fedorov.plainsvg.gui.actions.move.MoveUpAction;
import ru.amse.fedorov.plainsvg.model.ElementList;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.model.listeners.ElementChangedListener;
import ru.amse.fedorov.plainsvg.model.listeners.ModelChangedListener;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;
import ru.amse.fedorov.plainsvg.presentation.markers.Marker;
import ru.amse.fedorov.plainsvg.tools.CreateEllipseTool;
import ru.amse.fedorov.plainsvg.tools.CreateLineTool;
import ru.amse.fedorov.plainsvg.tools.CreatePolygonTool;
import ru.amse.fedorov.plainsvg.tools.CreateRectangleTool;
import ru.amse.fedorov.plainsvg.tools.SelectionTool;
import ru.amse.fedorov.plainsvg.tools.Tool;

/**
 * Component for drawing and redaction SVG image. Multifunctional and most
 * important part of the Plain SVG.
 * 
 * @author Alexey Fedorov
 */
public class GraphicsComponent extends JPanel {

	private static final long serialVersionUID = 1;

	private final int COMP_WIDTH = 200;

	private final int COMP_HEIGHT = 200;

	private double realWidth = COMP_WIDTH;

	private double realHeight = COMP_HEIGHT;

	private boolean imageChanged;

	private static final double SCALE_STEP = 1.5;

	private static final Color BORDER_COLOR = Color.BLACK;

	private final BasicStroke borderStroke = new BasicStroke(5);

	private final NotifyingPoint mousePoint = new NotifyingPoint();

	private final ElementList<ElementPresentation<? extends SVGElement>> elementList;

	private final ElementList<Marker<?>> markerList = new ElementList<Marker<?>>();

	private double scale;

	private final int shapeBorder = 2;

	private final ComponentSelectedList selectedList;

	private Tool currentTool;

	private SuperListener superListener = new SuperListener();

	private final Rectangle2D tempSelectionRect = new ListeningRectangle();

	private final ComponentBorder border = new ComponentBorder();

	private final Tool selectTool;

	private final Tool createRectTool;

	private final Tool createEllipseTool;

	private final Tool createLineTool;

	private final Tool createPolygonTool;

	private final ChangeToolAction selectAction;

	private final ChangeToolAction addRectAction;

	private final ChangeToolAction addEllipseAction;

	private final ChangeToolAction addLineAction;

	private final ChangeToolAction addPolygonAction;

	private final Action newAction;

	private final Action openAction;

	private final Action saveAction;

	private final Action saveAsAction;

	private final Action zoomIn;

	private final Action zoomOut;

	private final MoveAction moveUp;

	private final MoveAction moveDown;

	private final MoveAction moveTop;

	private final MoveAction moveBottom;

	private File file = null;

	private final PropertiesDialog propertiesDialog;

	/**
	 * Creates graphics panel with fixed data model (list of SVG elements) and
	 * fixed start scale on background panel.
	 * 
	 * @param list
	 *            list of SVG elements - data model
	 * @param scale
	 *            Start image scale
	 * @param backgroundPanel
	 *            background panel
	 */
	public GraphicsComponent(ElementList<ElementPresentation<?>> list,
			double scale) {
		setSize(COMP_WIDTH, COMP_HEIGHT);
		setDims(scale);
		imageChanged = false;
		setBackground(Color.WHITE);
		elementList = list;
		selectedList = new ComponentSelectedList(markerList);
		this.scale = scale;
		addListeners();

		newAction = new NewDocumentAction(this);
		openAction = new OpenDocumentAction(this);
		saveAction = new SaveDocumentAction(this);
		saveAsAction = new SaveAsDocumentAction(this);

		selectTool = new SelectionTool(this);
		createRectTool = new CreateRectangleTool(this);
		createEllipseTool = new CreateEllipseTool(this);
		createLineTool = new CreateLineTool(this);
		createPolygonTool = new CreatePolygonTool(this);

		moveUp = new MoveUpAction(this);
		moveDown = new MoveDownAction(this);
		moveTop = new MoveTopAction(this);
		moveBottom = new MoveBottomAction(this);

		selectedList.addSelectionChangedListener(moveUp);
		selectedList.addSelectionChangedListener(moveDown);
		selectedList.addSelectionChangedListener(moveTop);
		selectedList.addSelectionChangedListener(moveBottom);

		selectAction = new ChangeToolAction(this, selectTool, "Selection");

		addRectAction = new ChangeToolAction(this, createRectTool,
				"Add Rectangle", 'R');
		addEllipseAction = new ChangeToolAction(this, createEllipseTool,
				"Add Ellipse", 'E');
		addLineAction = new ChangeToolAction(this, createLineTool, "Add Line",
				'L');
		addPolygonAction = new ChangeToolAction(this, createPolygonTool,
				"Add Polygon", 'P');

		currentTool = selectTool;

		zoomIn = new ZoomAction(this, SCALE_STEP, "Zoom In", "zoom in", '1');
		zoomOut = new ZoomAction(this, 1 / SCALE_STEP, "Zoom Out", "zoom out",
				'2', 5);
		propertiesDialog = new PropertiesDialog(null, this);
	}

	/*
	 * Adds listeners.
	 */
	private void addListeners() {
		addMouseMotionListener(superListener);
		addMouseListener(superListener);
		selectedList.addSelectionChangedListener(superListener);
		for (ElementPresentation<? extends SVGElement> elem : elementList) {
			elem.getModelElement().addElementChangedListener(superListener);
		}
		addComponentListener(border);
	}

	/**
	 * Creates graphics panel with fixed data model (list of SVG elements) and
	 * default scale (default scale = 1) on background panel.
	 * 
	 * @param list
	 *            list of SVG elements - data model
	 * @param backgroundPanel
	 *            background panel
	 */
	public GraphicsComponent(ElementList<ElementPresentation<?>> list) {
		this(list, 1);
	}

	/**
	 * Creates graphics panel with empty data model (list of SVG elements) and
	 * default scale (default scale = 1) on background panel.
	 * 
	 * @param backgroundPanel
	 *            background panel
	 */
	public GraphicsComponent() {
		this(new ElementList<ElementPresentation<?>>(), 1);
	}

	/**
	 * Draws all SVG elements and minimal rectangle contained selected
	 * elements.In addition, marks specific points on selection rectangle's
	 * border, if need.
	 * 
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D graphics2D = (Graphics2D) g;
		AffineTransform oldTransform = graphics2D.getTransform();
		tuneGraphics(graphics2D);
		drawElements(graphics2D);

		if (selectedList.isEmpty()) {
			drawSelectionRectangle(graphics2D);
		}

		for (Marker<?> marker : markerList) {
			marker.paint(graphics2D, scale);
		}

		graphics2D.setTransform(oldTransform);
		graphics2D.setStroke(borderStroke);
		graphics2D.setPaint(BORDER_COLOR);
		graphics2D.draw(border);
	}

	/*
	 * Draws rectangle for selection.
	 */
	private void drawSelectionRectangle(Graphics2D graphics2D) {
		AffineTransform oldTransform = graphics2D.getTransform();
		AffineTransform at = new AffineTransform();
		at.setToScale(scale, scale);
		graphics2D.transform(at);
		float s = (float) getScale();
		float[] p = { 3 / s, 2 * 3 / s };
		BasicStroke bs = new BasicStroke(1.0f / s, BasicStroke.CAP_SQUARE,
				BasicStroke.JOIN_MITER, 10.0f, p, 0.0f);
		graphics2D.setStroke(bs);
		graphics2D.setPaint(Color.BLACK);
		graphics2D.setPaint(Color.BLACK);
		graphics2D.draw(tempSelectionRect);
		graphics2D.setTransform(oldTransform);
	}

	/*
	 * Draws all elements.
	 * 
	 * graphics2D - graphics to draw
	 */
	private void drawElements(Graphics2D graphics2D) {
		AffineTransform oldTransform = graphics2D.getTransform();
		AffineTransform scaledTransform = new AffineTransform();
		for (ElementPresentation<? extends SVGElement> elem : elementList) {
			scaledTransform.setToScale(scale, scale);
			AffineTransform at = elem.getTransform();
			scaledTransform.concatenate(at);
			graphics2D.transform(scaledTransform);
			elem.paint(graphics2D, scale);
			graphics2D.setTransform(oldTransform);
		}
	}

	/*
	 * Tunes graphics effects.
	 * 
	 * graphics2D - tuned graphics
	 */
	private void tuneGraphics(Graphics2D graphics2D) {
		// Mounts graphics effects

		graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		graphics2D.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
				RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);

		graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING,
				RenderingHints.VALUE_COLOR_RENDER_QUALITY);
	}

	/**
	 * @return Current image's scale
	 */
	public double getScale() {
		return scale;
	}

	/**
	 * @param scale
	 *            new scale
	 */
	public void scale(double zoom) {
		scale = scale * zoom;
		setDims(zoom);
		repaint();
	}

	private void setDims(double zoom) {
		Dimension dim = new Dimension((int) (getWidth() * zoom),
				(int) (getHeight() * zoom));
		setSize(dim);
		setPreferredSize(dim);
	}

	/**
	 * @return list of SVG elements
	 */
	public ElementList<ElementPresentation<?>> getElementList() {
		return elementList;
	}

	/**
	 * @return the tempSelectionRect
	 */
	public Rectangle2D getTempSelectionRect() {
		return tempSelectionRect;
	}

	/**
	 * @return the selectedList
	 */
	public ComponentSelectedList getSelectedList() {
		return selectedList;
	}

	public SuperListener getSuperListener() {
		return superListener;
	}

	public ChangeToolAction getSelectAction() {
		return selectAction;
	}

	/**
	 * @return the addRectAction
	 */
	public ChangeToolAction getAddRectAction() {
		return addRectAction;
	}

	/**
	 * @return the addEllipseAction
	 */
	public ChangeToolAction getAddEllipseAction() {
		return addEllipseAction;
	}

	/**
	 * @return the addLineAction
	 */
	public ChangeToolAction getAddLineAction() {
		return addLineAction;
	}

	/**
	 * @return the addLineAction
	 */
	public ChangeToolAction getAddPolygonAction() {
		return addPolygonAction;
	}

	/**
	 * @return the zoomIn
	 */
	public Action getZoomIn() {
		return zoomIn;
	}

	/**
	 * @return the zoomOut
	 */
	public Action getZoomOut() {
		return zoomOut;
	}

	/**
	 * @return the moveUp
	 */
	public Action getMoveUp() {
		return moveUp;
	}

	/**
	 * @return the moveDown
	 */
	public Action getMoveDown() {
		return moveDown;
	}

	/**
	 * @return the moveTop
	 */
	public Action getMoveTop() {
		return moveTop;
	}

	/**
	 * @return the moveBottom
	 */
	public Action getMoveBottom() {
		return moveBottom;
	}

	/**
	 * @return the mousePoint
	 */
	public NotifyingPoint getMousePoint() {
		return mousePoint;
	}

	/**
	 * Mouse adapter for graphics component. All MouseAdapter readresses to
	 * current tool.
	 * 
	 * @author Alexey Fedorov
	 */
	private final class SuperListener implements MouseListener,
			MouseMotionListener, ModelChangedListener,
			SelectionChangedListener, ElementChangedListener {

		public Point2D getConvertedPoint(MouseEvent e) {
			return new Point2D.Double(e.getX() / scale, e.getY() / scale);
		}

		/**
		 * Readresses mouse click to current tool.
		 */
		public void mouseClicked(MouseEvent e) {
			currentTool.mouseClicked(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse enter to current tool.
		 */
		public void mouseEntered(MouseEvent e) {
			currentTool.mouseEntered(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse exit to current tool.
		 */
		public void mouseExited(MouseEvent e) {
			currentTool.mouseExited(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse press to current tool.
		 */
		public void mousePressed(MouseEvent e) {
			currentTool.mousePressed(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse release to current tool.
		 */
		public void mouseReleased(MouseEvent e) {
			currentTool.mouseReleased(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse drag to current tool.
		 */
		public void mouseDragged(MouseEvent e) {
			mousePoint.setLocation(getConvertedPoint(e));
			currentTool.mouseDragged(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Readresses mouse's moving to current tool.
		 */
		public void mouseMoved(MouseEvent e) {
			mousePoint.setLocation(getConvertedPoint(e));
			currentTool.mouseMoved(getConvertedPoint(e), e.isControlDown());
		}

		/**
		 * Repaints component is model has been changed.
		 */
		public void modelChanged() {
			imageChanged = true;
			repaint();
		}

		/**
		 * Repaints component if selection has been changed.
		 */
		public void selectionChanged() {
			repaint();
		}

		/**
		 * Repaints component if some elements has been changed.
		 */
		public void elementChanged() {
			imageChanged = true;
			repaint();
		}

		@Override
		public String toString() {
			return "Super-puper listener";
		}

	}

	/**
	 * @return the currentTool
	 */
	public Tool getCurrentTool() {
		return currentTool;
	}

	/**
	 * @param currentTool
	 *            the currentTool to set
	 */
	public void setCurrentTool(Tool currentTool) {
		getCurrentTool().setCurrent(false);
		this.currentTool = currentTool;
		getCurrentTool().setCurrent(true);
	}

	public void setDefaultCurrentTool() {
		setCurrentTool(selectTool);
	}

	private class ListeningRectangle extends Rectangle2D {

		private Rectangle2D rect = new Rectangle2D.Double();

		@Override
		public Rectangle2D createIntersection(Rectangle2D r) {
			return rect.createIntersection(r);
		}

		@Override
		public Rectangle2D createUnion(Rectangle2D r) {
			return rect.createUnion(r);
		}

		@Override
		public int outcode(double x, double y) {
			return rect.outcode(x, y);
		}

		@Override
		public void setRect(double x, double y, double w, double h) {
			rect.setRect(x, y, w, h);
			repaint();
		}

		@Override
		public double getHeight() {
			return rect.getHeight();
		}

		@Override
		public double getWidth() {
			return rect.getWidth();
		}

		@Override
		public double getX() {
			return rect.getX();
		}

		@Override
		public double getY() {
			return rect.getY();
		}

		@Override
		public boolean isEmpty() {
			return rect.isEmpty();
		}

	}

	/**
	 * @return the markerList
	 */
	public ElementList<Marker<?>> getMarkerList() {
		return markerList;
	}

	public void reload(List<ElementPresentation<?>> list, Element root) {
		String width = root.getAttribute("width");
		String height = root.getAttribute("height");
		width = width.substring(0, width.length() - 2);
		height = height.substring(0, height.length() - 2);
		double w = Double.parseDouble(width);
		double h = Double.parseDouble(height);
		scale = 1;
		setRealSize(w, h);
		reload(list);
	}

	public void reload(List<ElementPresentation<?>> list) {
		selectedList.clear();
		elementList.clear();
		scale = 1;
		elementList.addAll(list);
		imageChanged = false;
		setVisible(true);
	}

	public double getRealWidth() {
		return realWidth;
	}

	public double getRealHeight() {
		return realHeight;
	}

	/**
	 * @return the file
	 */
	public File getFile() {
		return file;
	}

	/**
	 * @param file
	 *            the file to set
	 */
	public void setFile(File file) {
		this.file = file;
	}

	/**
	 * @return the changed
	 */
	public boolean isModelChanged() {
		return imageChanged;
	}

	/**
	 * @param modelChanged
	 *            the modelChanged to set
	 */
	public void setModelChanged(boolean modelChanged) {
		this.imageChanged = modelChanged;
	}

	/**
	 * @return the propertiesDialog
	 */
	public PropertiesDialog getPropertiesDialog() {
		return propertiesDialog;
	}

	private class ComponentBorder extends Rectangle2D implements
			ComponentListener {

		private final Rectangle2D rect = new Rectangle2D.Double(0, 0,
				COMP_WIDTH - shapeBorder / 2, COMP_HEIGHT - shapeBorder / 2);

		@Override
		public Rectangle2D createIntersection(Rectangle2D r) {
			return rect.createIntersection(r);
		}

		@Override
		public Rectangle2D createUnion(Rectangle2D r) {
			return rect.createUnion(r);
		}

		@Override
		public int outcode(double x, double y) {
			return rect.outcode(x, y);
		}

		@Override
		public void setRect(double x, double y, double w, double h) {
			rect.setRect(x, y, w, h);
		}

		@Override
		public double getHeight() {
			return rect.getHeight();
		}

		@Override
		public double getWidth() {
			return rect.getWidth();
		}

		@Override
		public double getX() {
			return rect.getX();
		}

		@Override
		public double getY() {
			return rect.getY();
		}

		@Override
		public boolean isEmpty() {
			return rect.isEmpty();
		}

		public void componentHidden(ComponentEvent e) {
			// Nothing!
		}

		public void componentMoved(ComponentEvent e) {
			// Nothing!
		}

		public void componentResized(ComponentEvent e) {
			int w = (int) (GraphicsComponent.this.getWidth() - shapeBorder / 2);
			int h = (int) (GraphicsComponent.this.getHeight() - shapeBorder / 2);
			rect.setFrame(0, 0, w, h);
		}

		public void componentShown(ComponentEvent e) {
			// Nothing!
		}

	}

	public void setRealSize(double w, double h) {
		imageChanged = true;
		realWidth = w;
		realHeight = h;
		Dimension dim = new Dimension((int) (w * scale), (int) (h * scale));
		setSize(dim);
		setPreferredSize(dim);
	}

	/**
	 * @return the newAction
	 */
	public Action getNewAction() {
		return newAction;
	}

	/**
	 * @return the openAction
	 */
	public Action getOpenAction() {
		return openAction;
	}

	/**
	 * @return the saveAction
	 */
	public Action getSaveAction() {
		return saveAction;
	}

	/**
	 * @return the saveAsAction
	 */
	public Action getSaveAsAction() {
		return saveAsAction;
	}
}
