package ru.amse.fedorov.plainsvg.model;

import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;

/**
 * Expanded class for affine transforms. Every modifying method of this
 * transform notifies listeners by means of method <b>fireContentChanged()</b>
 * 
 * @author Alexey Fedorov
 * 
 * @see java.awt.geom.AffineTransform
 */
public abstract class NotifyingAffineTransform extends AffineTransform {

	private static final long serialVersionUID = 1L;

	public NotifyingAffineTransform() {
		super();
	}
	
	/*
	 * This method norifies that transform has been changed.
	 */
	protected abstract void fireContentChanged();

	@Override
	public void concatenate(AffineTransform Tx) {
		super.concatenate(Tx);
		fireContentChanged();
	}

	@Override
	public void setToIdentity() {
		super.setToIdentity();
		fireContentChanged();
	}

	@Override
	public void setToQuadrantRotation(int numquadrants, double anchorx,
			double anchory) {
		super.setToQuadrantRotation(numquadrants, anchorx, anchory);
		fireContentChanged();
	}

	@Override
	public void setToQuadrantRotation(int numquadrants) {
		super.setToQuadrantRotation(numquadrants);
		fireContentChanged();
	}

	@Override
	public void setToRotation(double vecx, double vecy, double anchorx,
			double anchory) {
		super.setToRotation(vecx, vecy, anchorx, anchory);
		fireContentChanged();
	}

	@Override
	public void setToRotation(double theta, double anchorx, double anchory) {
		super.setToRotation(theta, anchorx, anchory);
		fireContentChanged();
	}

	@Override
	public void setToRotation(double vecx, double vecy) {
		super.setToRotation(vecx, vecy);
		fireContentChanged();
	}

	@Override
	public void setToRotation(double theta) {
		super.setToRotation(theta);
		fireContentChanged();
	}

	@Override
	public void setToScale(double sx, double sy) {
		super.setToScale(sx, sy);
		fireContentChanged();
	}

	@Override
	public void setToShear(double shx, double shy) {
		super.setToShear(shx, shy);
		fireContentChanged();
	}

	@Override
	public void setToTranslation(double tx, double ty) {
		super.setToTranslation(tx, ty);
		fireContentChanged();
	}

	@Override
	public void shear(double shx, double shy) {
		super.shear(shx, shy);
		fireContentChanged();
	}

	@Override
	public void transform(double[] srcPts, int srcOff, double[] dstPts,
			int dstOff, int numPts) {
		super.transform(srcPts, srcOff, dstPts, dstOff, numPts);
		fireContentChanged();
	}

	@Override
	public void translate(double tx, double ty) {
		super.translate(tx, ty);
		fireContentChanged();
	}

	@Override
	public void invert() {
		try {
			super.invert();
		} catch (NoninvertibleTransformException e) {
			assert false;
		}
		fireContentChanged();
	}

	@Override
	public void rotate(double vecx, double vecy, double anchorx, double anchory) {
		super.translate(anchorx, anchory);
		super.rotate(vecx, vecy);
		super.translate(-anchorx, -anchory);
		fireContentChanged();
	}

	@Override
	public void rotate(double theta, double anchorx, double anchory) {
		super.translate(anchorx, anchory);
		super.rotate(theta);
		super.translate(-anchorx, -anchory);
		fireContentChanged();
	}

	@Override
	public void rotate(double vecx, double vecy) {
		super.rotate(vecx, vecy);
		fireContentChanged();
	}

	@Override
	public void rotate(double theta) {
		super.rotate(theta);
		fireContentChanged();
	}

	@Override
	public void scale(double sx, double sy) {
		super.scale(sx, sy);
		fireContentChanged();
	}

	@Override
	public void inverseTransform(double[] srcPts, int srcOff, double[] dstPts,
			int dstOff, int numPts) throws NoninvertibleTransformException {
		try {
			super.inverseTransform(srcPts, srcOff, dstPts, dstOff, numPts);
		} catch (NoninvertibleTransformException e) {
			assert false;
		}
	}

	@Override
	public Point2D inverseTransform(Point2D ptSrc, Point2D ptDst) {
		try {
			return super.inverseTransform(ptSrc, ptDst);
		} catch (NoninvertibleTransformException e) {
			assert false;
		}
		return ptSrc;
	}

}
