package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.model.NotifyingAffineTransform;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.presentation.Presentation;
import ru.amse.fedorov.plainsvg.presentation.markers.Marker;

/**
 * Wraps 2d-element ang svg-element and realises its' connection.
 * 
 * @author Alexey Fedorov
 */
public abstract class ElementPresentation<T extends SVGElement> extends
		Presentation {

	private final List<Marker<?>> list = new ArrayList<Marker<?>>();

	/**
	 * 
	 * If one of svg-elements contains point, in which mouse had been clicked,
	 * its' minimal circumscribing rectangle.
	 * 
	 * @return minimal circumscribing rectangle with sides encamping parallel
	 *         with system axes selects
	 */
	public abstract Rectangle2D getMinRect();

	/**
	 * @return 3x2-matrix of corresponding svg-element, describes affine
	 *         transformation
	 */
	public NotifyingAffineTransform getTransform() {
		return getModelElement().getTransform();
	}

	/**
	 * @param matrix
	 *            3x2-matrix of corresponding svg-element, describes affine
	 *            transformation
	 */
	public void setTransform(AffineTransform at) {
		getModelElement().setAffineTransform(at);
	}

	/**
	 * Returns <b>true</b> if this element has stroke and <b>false</b> if not.
	 */
	public abstract boolean hasStroke();

	/**
	 * Returns <b>true</b> if this element has fill and <b>false</b> if not.
	 */
	public abstract boolean hasFill();

	@Override
	public String toString() {
		return "Presentation of svg-" + getModelElement().toString();
	}

	public Collection<Marker<?>> getMarkers() {
		return list;
	}

	public final void addMarkers() {
		addAllMarkers();
		for (Marker<?> m : getMarkers()) {
			m.arrangeLocation();
		}
	}
	
	protected abstract void addAllMarkers();


	public final Element getElement(Document document) {
		Element element = document.createElement(getName());
		createElement(element);
		element.setAttribute("transform", getMatrixString());
		return element;
	}

	protected abstract void createElement(Element element);

	protected abstract String getName();

	/**
	 * Determines if element contains point.
	 * 
	 * @param point
	 *            to determine
	 * 
	 * @return <b>true</b> if current element contains point with <b>x</b> and
	 *         <b>y</b> coordinates and <b>false</b> if not
	 */
	public final boolean contains(Point2D point, double scale) {
		Point2D untransformed = new Point2D.Double();
		untransformed = getTransform().inverseTransform(point, untransformed);
		return containsUntransformed(untransformed, scale);
	}

	/**
	 * Determines if element in his start configuration (before affine
	 * transforming) contains point with stated coordinates. Calls from
	 * <b>contains</b>.
	 * 
	 * @return <b>true</b> if current element contains point and <b>false</b>
	 *         if not
	 */
	protected abstract boolean containsUntransformed(Point2D p, double scale);

	public String getMatrixString() {
		StringBuffer buffer = new StringBuffer();
		AffineTransform at = getTransform();
		buffer.append("matrix(");
		buffer.append(at.getScaleX() + " ");
		buffer.append(at.getShearY() + " ");
		buffer.append(at.getShearX() + " ");
		buffer.append(at.getScaleY() + " ");
		buffer.append(at.getTranslateX() + " ");
		buffer.append(at.getTranslateY() + " ");
		buffer.append(')');
		return buffer.toString();
	}

	/**
	 * @return svg-shape, wrapped in this element
	 */
	public abstract T getModelElement();

}
