package ru.amse.fedorov.plainsvg.presentation.markers;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.model.listeners.ElementChangedListener;
import ru.amse.fedorov.plainsvg.presentation.Presentation;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

public abstract class Marker<E extends ElementPresentation<?>> extends
		Presentation implements ElementChangedListener {

	private static final double RADIUS = 3;

	private final Ellipse2D circle = new Ellipse2D.Double();

	private final E presentation;

	private final Color color;

	private final Cursor cursor;

	public Marker(E pres, Color color, Cursor cursor) {
		presentation = pres;
		presentation.getModelElement().addElementChangedListener(this);
		this.color = color;
		this.cursor = cursor;
	}

	public Marker(E pres) {
		this(pres, new Color(255, 150, 0), Cursor
				.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
	}

	protected Color getMarkerColor() {
		return color;
	}

	public Cursor getCursor() {
		return cursor;
	}

	@Override
	public Shape getShape() {
		return circle;
	}

	/**
	 * Draws marker.
	 * 
	 * @param g
	 *            graphics to draw
	 * @param scale
	 *            current component's scale
	 */
	@Override
	public void paint(Graphics2D g, double scale) {
		g.setStroke(new BasicStroke(1));
		g.setColor(getMarkerColor());
		arrangeLocation();
		double x = (circle.getMinX() + circle.getMaxX()) / 2;
		double y = (circle.getMinY() + circle.getMaxY()) / 2;
		double cx = x * scale;
		double cy = y * scale;
		circle.setFrame(cx - RADIUS, cy - RADIUS, 2 * RADIUS, 2 * RADIUS);
		g.fill(circle);
		g.setColor(Color.BLACK);
		g.draw(circle);
		circle.setFrame(x - RADIUS, y - RADIUS, 2 * RADIUS, 2 * RADIUS);
	}

	public void arrangeLocation() {
		Point2D p = getTransformedPoint();
		circle.setFrame(p.getX() - RADIUS, p.getY() - RADIUS, 2 * RADIUS,
				2 * RADIUS);
	}

	/**
	 * @return point, where marker has to be.
	 */
	protected abstract Point2D getPoint();

	private Point2D getTransformedPoint() {
		AffineTransform at = presentation.getTransform();
		return at.transform(getPoint(), null);
	}

	/**
	 * @return the circle
	 */
	public Ellipse2D getCircle() {
		return (Ellipse2D) circle.clone();
	}

	/**
	 * @return the presentation
	 */
	public E getPresentation() {
		return presentation;
	}

	public void elementChanged() {
		arrangeLocation();
	}

	public abstract void movedUtransformed(Point2D p);

	@Override
	public String toString() {
		Point2D p = getTransformedPoint();
		return "Marker[" + p.getX() + ", " + p.getY() + "]";
	}

	public final void moveMarker(Point2D point) {
		movedUtransformed(getPresentation().getTransform().inverseTransform(
				point, null));
	}

	protected double getMarkerRadius() {
		return RADIUS;
	}

	public boolean contains(Point2D point, double scale) {
		return getTransformedPoint().distance(point) < RADIUS / scale;
	}

}
