package ru.amse.fedorov.plainsvg.tools;

import java.awt.Cursor;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.NotifyingAffineTransform;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;
import ru.amse.fedorov.plainsvg.presentation.markers.Marker;

/**
 * An abstract adapter class for receiving mouse events. The methods in this
 * class are empty. Realises only one method: mouse clicked, which iterates list
 * of all svg-elements of pictures in top-down order. If one of svg-elements
 * contains point, in which mouse had been clicked, its' minimal circumscribing
 * rectangle with sides encamping parallel with system axes selects.
 * 
 * @author Alexey Fedorov
 */
public class SelectionTool extends ToolAdapter {

	/**
	 * The point when mouse was pressed.
	 */
	protected Point2D previousPoint = new Point2D.Double();

	private Rectangle2D selectionRectangle;

	private Point2D pointWherePressed = new Point2D.Double();

	private boolean pressedInMarkerList;

	private Marker<?> currentMarker;

	/*
	 * If current operation is selection with rectangle
	 */
	private boolean isSelectionState = false;

	/**
	 * Base constructor.
	 * 
	 * @param component
	 *            graphics component
	 */
	public SelectionTool(GraphicsComponent component) {
		super(component);
		selectionRectangle = component.getTempSelectionRect();
	}

	/**
	 * Iterated list of all svg-elements of pictures in top-down order. If one
	 * of svg-elements contains point, in which mouse had been clicked, its'
	 * minimal circumscribing rectangle with sides encamping parallel with
	 * system axes selects.
	 */
	@Override
	public void mousePressed(Point2D point, boolean isControlDown) {
		previousPoint.setLocation(point);
		pressedInMarkerList = isMarkerListContains(point);
		pointWherePressed.setLocation(point);
		ElementPresentation<?> elem;

		ElementPresentation<?> removed = null;
		if (isControlDown) {
			for (int i = getElementList().size(); i > 0; i--) {
				elem = getElementList().get(i - 1);
				if (elem.contains(point, getComponent().getScale())) {
					if (getSelectedList().contains(elem)) {
						removed = elem;
						getSelectedList().remove(elem);
					} else {
						getSelectedList().add(elem);
					}
					break;
				}
			}

		} else if ((!isSelectedListContains(point))
				&& (!isMarkerListContains(point))) {
			getSelectedList().clear();
		}

		isSelectionState = (!isElementListContains(point))
				&& (!isMarkerListContains(point));

		if (currentMarker == null) {
			if ((!isDirectUp(point)) && (!isSelectedListContains(point))) {
				getSelectedList().clear();
			}
			if (!isDirectUp(point) && !isControlDown) {
				boolean topContains = true;

				for (int i = getElementList().size(); i > 0; i--) {
					ElementPresentation<?> e = getElementList().get(i - 1);
					if (e.contains(point, getScale())) {
						topContains = getSelectedList().contains(e);
						break;
					}

				}
				if (!topContains) {
					getSelectedList().clear();
				}
			}
			for (int i = getElementList().size(); i > 0; i--) {
				elem = getElementList().get(i - 1);
				if (elem.contains(point, getScale())) {
					if (!getSelectedList().contains(elem) && (elem != removed)) {
						getSelectedList().add(elem);
					}
					break;
				}
			}
		}
		removed = null;
	}

	@Override
	public void mouseReleased(Point2D point, boolean isControlDown) {
		previousPoint.setLocation(point);
		if (isSelectionState) {
			isSelectionState = false;
			List<ElementPresentation<?>> list = new ArrayList<ElementPresentation<?>>();
			for (ElementPresentation<?> elem : getElementList()) {
				if (selectionRectangle.contains(elem.getMinRect())) {
					list.add(elem);
				}
			}
			for (ElementPresentation<?> elem : list) {
				getSelectedList().add(elem);
			}
			selectionRectangle.setFrame(0, 0, 0, 0);
		}
	}

	@Override
	public void mouseDragged(Point2D point, boolean isControlDown) {
		if (isSelectionState) {
			double x = Math.min(point.getX(), pointWherePressed.getX());
			double y = Math.min(point.getY(), pointWherePressed.getY());
			double w = Math.abs(point.getX() - pointWherePressed.getX());
			double h = Math.abs(point.getY() - pointWherePressed.getY());
			selectionRectangle.setFrame(x, y, w, h);
		} else {
			double x = point.getX() - previousPoint.getX();
			double y = point.getY() - previousPoint.getY();
			Point2D p = new Point2D.Double();
			if (!isPressedInMarkerList()) {
				for (ElementPresentation<? extends SVGElement> elem : getSelectedList()) {
					NotifyingAffineTransform at = elem.getTransform();
					p.setLocation(x + at.getTranslateX(), y
							+ at.getTranslateY());
					p = at.inverseTransform(p, null);
					at.translate(p.getX(), p.getY());
				}
			} else {
				getCurrentMarker().moveMarker(point);
			}
		}
		previousPoint.setLocation(point);
	}

	@Override
	public void mouseMoved(Point2D point, boolean isControlDown) {
		if (isMarkerListContains(point)) {
			getComponent().setCursor(getCurrentMarker().getCursor());
		} else if (isElementListContains(point)) {
			getComponent().setCursor(
					Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		} else {
			getComponent().setCursor(Cursor.getDefaultCursor());
		}
	}

	/**
	 * @return the pressedInMarkerList
	 */
	public boolean isPressedInMarkerList() {
		return pressedInMarkerList;
	}

	/**
	 * @return the currentMarker
	 */
	public Marker<?> getCurrentMarker() {
		return currentMarker;
	}

	protected boolean isMarkerListContains(Point2D point) {
		currentMarker = null;
		for (Marker<?> marker : getComponent().getMarkerList()) {
			if (marker.contains(point, getScale())) {
				currentMarker = marker;
				getComponent().getMarkerList().moveTop(currentMarker);
				return true;
			}
		}
		return false;
	}

}
