package ru.amse.fedorov.plainsvg.tools;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import ru.amse.fedorov.plainsvg.gui.ComponentSelectedList;
import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.ElementList;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

/**
 * The basic interface for all editor's tools. Every editor's tool implements
 * this interface. It is original analog for mouse listeners.
 * 
 * @author Alexey Fedorov
 */
public abstract class Tool {

	private ArrayList<ToolStatusListener> listeners = new ArrayList<ToolStatusListener>(
			2);

	/*
	 * TRUE if tool is current
	 */
	private boolean current;

	/*
	 * The component for drawing and redaction SVG image.
	 */
	private final GraphicsComponent component;

	/**
	 * 
	 * 
	 * @param component
	 */
	public Tool(GraphicsComponent component) {
		this.component = component;
	}

	/**
	 * @param point
	 *            The point, where the mouse was clicked
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseClicked(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse was dragged
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseDragged(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse was entered
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseEntered(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            T The point, where the mouse was exited
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseExited(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse was moved
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseMoved(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse was pressed
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mousePressed(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse was released
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseReleased(Point2D point, boolean isControlDown);

	/**
	 * @param point
	 *            The point, where the mouse's wheel was moved
	 * 
	 * @param isControlDown
	 *            TRUE if <i>CONTROL</i> key is pressed
	 */
	public abstract void mouseWheelMoved(Point2D point, boolean isControlDown);

	/**
	 * @return the component
	 */
	protected GraphicsComponent getComponent() {
		return component;
	}

	/**
	 * @return the selectedList
	 */
	protected ComponentSelectedList getSelectedList() {
		return component.getSelectedList();
	}

	protected boolean isSelectedListContains(Point2D point) {
		for (ElementPresentation<?> elem : getSelectedList()) {
			if (elem.contains(point, getScale())) {
				return true;
			}
		}
		return false;
	}

	protected boolean isElementListContains(Point2D point) {
		for (ElementPresentation<?> elem : getElementList()) {
			if (elem.contains(point, getScale())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return the elementList
	 */
	protected ElementList<ElementPresentation<?>> getElementList() {
		return component.getElementList();
	}

	/**
	 * @return the current
	 */
	public boolean isCurrent() {
		return current;
	}

	/**
	 * @param current
	 *            the current to set
	 */
	public void setCurrent(boolean current) {
		this.current = current;
		fireStatusChanged();
	}

	public void addToolStatusListener(ToolStatusListener listener) {
		listeners.add(listener);
	}

	public void removeToolStatusListener(ToolStatusListener listener) {
		listeners.remove(listener);
	}

	protected void fireStatusChanged() {
		for (ToolStatusListener listener : listeners) {
			listener.doOnStatusChanged();
		}
	}

	public double getScale() {
		return getComponent().getScale();
	}

	protected boolean isDirectUp(Point2D point) {
		if (! isElementListContains(point)) {
			return false;
		}
		for (ElementPresentation<?> elem : getElementList()) {
			if (elem.contains(point, getScale())
					&& (getSelectedList().contains(elem))) {
				return false;
			}
		}
		return true;
	}
}
